@@ -2,15 +2,15 @@
 from __future__ import unicode_literals
 
 from prompt_toolkit.enums import DEFAULT_BUFFER
-from prompt_toolkit.filters import CLIFilter, Always, HasSelection, Condition, EmacsInsertMode, ViInsertMode
+from prompt_toolkit.filters import HasSelection, Condition, EmacsInsertMode, ViInsertMode
 from prompt_toolkit.keys import Keys
 from prompt_toolkit.layout.screen import Point
 from prompt_toolkit.mouse_events import MouseEventType, MouseEvent
 from prompt_toolkit.renderer import HeightIsUnknownError
 from prompt_toolkit.utils import suspend_to_background_supported, is_windows
 
-from .utils import create_handle_decorator
 from .named_commands import get_by_name
+from ..registry import Registry
 
 
 __all__ = (
@@ -26,11 +26,10 @@ def if_no_repeat(event):
     return not event.is_repeat
 
 
-def load_basic_bindings(registry, filter=Always()):
-    assert isinstance(filter, CLIFilter)
-
+def load_basic_bindings():
+    registry = Registry()
     insert_mode = ViInsertMode() | EmacsInsertMode()
-    handle = create_handle_decorator(registry, filter)
+    handle = registry.add_binding
     has_selection = HasSelection()
 
     @handle(Keys.ControlA)
@@ -142,7 +141,8 @@ def load_basic_bindings(registry, filter=Always()):
         get_by_name('self-insert'))
     handle(Keys.ControlT, filter=insert_mode)(get_by_name('transpose-chars'))
     handle(Keys.ControlW, filter=insert_mode)(get_by_name('unix-word-rubout'))
-    handle(Keys.ControlI, filter=insert_mode)(get_by_name('complete'))
+    handle(Keys.ControlI, filter=insert_mode)(get_by_name('menu-complete'))
+    handle(Keys.BackTab, filter=insert_mode)(get_by_name('menu-complete-backward'))
 
     handle(Keys.PageUp, filter= ~has_selection)(get_by_name('previous-history'))
     handle(Keys.PageDown, filter= ~has_selection)(get_by_name('next-history'))
@@ -152,13 +152,6 @@ def load_basic_bindings(registry, filter=Always()):
     text_before_cursor = Condition(lambda cli: cli.current_buffer.text)
     handle(Keys.ControlD, filter=text_before_cursor & insert_mode)(get_by_name('delete-char'))
 
-    @handle(Keys.BackTab, filter=insert_mode)
-    def _(event):
-        """
-        Shift+Tab: go to previous completion.
-        """
-        event.current_buffer.complete_previous()
-
     is_multiline = Condition(lambda cli: cli.current_buffer.is_multiline())
     is_returnable = Condition(lambda cli: cli.current_buffer.accept_action.is_returnable)
 
@@ -188,7 +181,7 @@ def load_basic_bindings(registry, filter=Always()):
         data = event.current_buffer.cut_selection()
         event.cli.clipboard.set_data(data)
 
-    # Global bindings. These are never disabled and don't include the default filter.
+    # Global bindings.
 
     @handle(Keys.ControlZ)
     def _(event):
@@ -203,7 +196,7 @@ def load_basic_bindings(registry, filter=Always()):
         """
         event.current_buffer.insert_text(event.data)
 
-    @registry.add_binding(Keys.CPRResponse)
+    @handle(Keys.CPRResponse, save_before=lambda e: False)
     def _(event):
         """
         Handle incoming Cursor-Position-Request response.
@@ -215,7 +208,7 @@ def load_basic_bindings(registry, filter=Always()):
         # Report absolute cursor position to the renderer.
         event.cli.renderer.report_absolute_cursor_row(row)
 
-    @registry.add_binding(Keys.BracketedPaste)
+    @handle(Keys.BracketedPaste)
     def _(event):
         " Pasting from clipboard. "
         data = event.data
@@ -228,12 +221,24 @@ def load_basic_bindings(registry, filter=Always()):
 
         event.current_buffer.insert_text(data)
 
+    @handle(Keys.Any, filter=Condition(lambda cli: cli.quoted_insert), eager=True)
+    def _(event):
+        """
+        Handle quoted insert.
+        """
+        event.current_buffer.insert_text(event.data, overwrite=False)
+        event.cli.quoted_insert = False
 
-def load_mouse_bindings(registry, filter=Always()):
+    return registry
+
+
+def load_mouse_bindings():
     """
     Key bindings, required for mouse support.
     (Mouse events enter through the key binding system.)
     """
+    registry = Registry()
+
     @registry.add_binding(Keys.Vt100MouseEvent)
     def _(event):
         """
@@ -330,13 +335,15 @@ def load_mouse_bindings(registry, filter=Always()):
         handler(event.cli, MouseEvent(position=Point(x=x, y=y),
                                       event_type=event_type))
 
+    return registry
 
-def load_abort_and_exit_bindings(registry, filter=Always()):
+
+def load_abort_and_exit_bindings():
     """
     Basic bindings for abort (Ctrl-C) and exit (Ctrl-D).
     """
-    assert isinstance(filter, CLIFilter)
-    handle = create_handle_decorator(registry, filter)
+    registry = Registry()
+    handle = registry.add_binding
 
     @handle(Keys.ControlC)
     def _(event):
@@ -352,31 +359,34 @@ def load_abort_and_exit_bindings(registry, filter=Always()):
 
     handle(Keys.ControlD, filter=ctrl_d_condition)(get_by_name('end-of-file'))
 
+    return registry
 
-def load_basic_system_bindings(registry, filter=Always()):
+
+def load_basic_system_bindings():
     """
     Basic system bindings (For both Emacs and Vi mode.)
     """
-    assert isinstance(filter, CLIFilter)
-    handle = create_handle_decorator(registry, filter)
+    registry = Registry()
 
     suspend_supported = Condition(
         lambda cli: suspend_to_background_supported())
 
-    @handle(Keys.ControlZ, filter=suspend_supported)
+    @registry.add_binding(Keys.ControlZ, filter=suspend_supported)
     def _(event):
         """
         Suspend process to background.
         """
         event.cli.suspend_to_background()
 
+    return registry
 
-def load_auto_suggestion_bindings(registry, filter=Always()):
+
+def load_auto_suggestion_bindings():
     """
     Key bindings for accepting auto suggestion text.
     """
-    assert isinstance(filter, CLIFilter)
-    handle = create_handle_decorator(registry, filter)
+    registry = Registry()
+    handle = registry.add_binding
 
     suggestion_available = Condition(
         lambda cli:
@@ -393,3 +403,5 @@ def load_auto_suggestion_bindings(registry, filter=Always()):
 
         if suggestion:
             b.insert_text(suggestion.text)
+
+    return registry
